import random
from intern.zeichenfenster import *
from graphics_and_games_klassen import Rechteck, Text

class Patient:
    """Beschreibung der Klasse Patient"""
    
    def __init__(self, name):
        """Konstruktor des Patienten"""
        self.name = name
        self.nachfolger = None

    def NachfolgerSetzen(self, nf):
        """Nachfolger des Patienten setzen"""
        self.nachfolger = nf

    def NachfolgerGeben(self):
        """Gibt den Nachfolger des Patienten aus"""
        return self.nachfolger

    def InformationAusgeben(self):
        """Gibt Information über den Patienten aus"""
        print(self.name)

    def NameGeben(self):
        """Gibt den Namen des Patienten aus"""
        return self.name


class Warteschlange:
    """Beschreibung der Warteschlange"""
    def __init__(self): 
        """Konstruktor der Warteschlange"""
        self.anfang = None
        self.ende = None

    def Einfügen(self, patient_neu):
        """Fügt einen neuen Patienten am Ende der Warteschlange ein"""
        if self.anfang is None:
            self.anfang = patient_neu
        else:
            self.ende.NachfolgerSetzen(patient_neu)
        self.ende = patient_neu

    def Entfernen(self):
        """Entfernt den ersten Patient aus der Warteschlange und gibt ihn zurück"""
        p = self.anfang
        if self.anfang is not None:
            if self.anfang == self.ende:
                self.anfang = None
                self.ende = None
            else:
                self.anfang = self.anfang.NachfolgerGeben()
        return p

    def InformationAusgeben(self):
        """Gibt Information über die Warteschlange aus"""
        if self.anfang is not None:
            self.anfang.InformationAusgeben()
            
    def AnfangGeben(self):
        """Gibt den Anfang zurück"""
        return self.anfang


class PatientenSymbol:
    """
    Klasse für die Darstellung eines Patienten
    """
    farben = ["weiss", "rot", "gruen", "blau", "gelb", "cyan", "magenta", "braun", "orange", "grau"]

    def __init__(self):
        self.außen = Rechteck()
        self.außen.SichtbarkeitSetzen(False)
        self.außen.PositionSetzen(0, 0)
        self.außen.GroesseSetzen(100, 40)
        self.außen.FarbeSetzen("schwarz")
        self.innen = Rechteck()
        self.innen.SichtbarkeitSetzen(False)
        self.innen.PositionSetzen(2, 2)
        self.innen.GroesseSetzen(96, 36)
        self.innen.FarbeSetzen(random.choice(self.farben))
        self.name = Text()
        self.name.SichtbarkeitSetzen(False)
        self.name.PositionSetzen(4, 24)
        self.name.TextSetzen("")

    def PositionSetzen(self, x, y):
        self.außen.PositionSetzen(x, y)
        self.innen.PositionSetzen(x + 2, y + 2)
        self.name.PositionSetzen(x + 4, y + 24)

    def SichtbarkeitSetzen(self, sichtbar):
        self.außen.SichtbarkeitSetzen(sichtbar)
        self.innen.SichtbarkeitSetzen(sichtbar)
        self.name.SichtbarkeitSetzen(sichtbar)

    def TextSetzen(self, text):
        self.name.TextSetzen(text)

class Darstellung:
    """
    Stellt die Warteschlange im Wartezimmer dar
    """
    def __init__(self, ws):
        self.warteschlange = ws
        self.symbole = []

    def Anzeigen(self):
        self.KnotenAnzeigen(self.warteschlange.AnfangGeben(), 0)

    def KnotenAnzeigen(self, aktuell, nummer):
        if aktuell is not None:
            if len(self.symbole) <= nummer:
                self.symbole.append(PatientenSymbol())
            p = self.symbole[nummer]
            p.TextSetzen(aktuell.NameGeben())
            p.SichtbarkeitSetzen(True)
            b = 700
            h = 600
            if nummer < 7:
                x = nummer * 110
                y = 0
            elif nummer < 13:
                x = (nummer - 7) * 110
                y = 50
            elif nummer < 19:
                x = (nummer - 13) * 110
                y = 100
            elif nummer < 25:
                x = (nummer - 19) * 110
                y = 150
            else:
                p.SichtbarkeitSetzen(false)
                x = -200
                y = -100
            p.PositionSetzen(x, y)
            self.KnotenAnzeigen(aktuell.NachfolgerGeben(), nummer + 1)
        else:
            for i in range(nummer, len(self.symbole)):
                self.symbole[i].SichtbarkeitSetzen(False)

wartesch1 = Warteschlange()
patient1 = Patient("Andi")
patient2 = Patient("Berin")
patient3 = Patient("Carla")
patient4 = Patient("Didi")
patient5 = Patient("Emil")
patient6 = Patient("Franzi")
patient7 = Patient("Gabi")
patient8 = Patient("Hugo")
patient9 = Patient("Ida")
wartesch1.Einfügen(patient1)
wartesch1.Einfügen(patient2)
wartesch1.Einfügen(patient3)
wartesch1.Einfügen(patient4)
wartesch1.Einfügen(patient5)
wartesch1.Einfügen(patient6)
wartesch1.Einfügen(patient7)
wartesch1.Einfügen(patient8)
wartesch1.Einfügen(patient9)
#Die beiden folgenden Zeilen müssen immer die letzten sein (vorher kein Aufruf von Anzeigen!)
darstell1 = Darstellung(wartesch1)
darstell1.Anzeigen()

